/*         ______   ___    ___ 
 *        /\  _  \ /\_ \  /\_ \ 
 *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___ 
 *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
 *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
 *           \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/
 *            \/_/\/_/\/____/\/____/\/____/\/___L\ \/_/ \/___/
 *                                           /\____/
 *                                           \_/__/
 *      By Shawn Hargreaves,
 *      1 Salisbury Road,
 *      Market Drayton,
 *      Shropshire,
 *      England, TF9 1AJ.
 *
 *      Interpolation routines for 15 bit truecolor pixels.
 *
 *      See readme.txt for copyright information.
 */


#include "allegro.h"

#ifdef ALLEGRO_COLOR16

union dword { unsigned long l; struct { unsigned short lo,hi; } w; };

/* macro for constructing the blender routines */
#define BLEND(n)                                                         \
                                         \
    static unsigned long blend15_##n(union dword x, union dword y)       \
    {                                                                    \
    const unsigned long mask=0x3e07c1f;                                  \
    union dword result;                                                  \
    x.w.hi=x.w.lo,y.w.hi=y.w.lo;                                         \
    result.l=(((x.l&mask)*(n*32/31)+(y.l&mask)*(32-(n*32/31)))/32)&mask; \
    return (result.w.hi|result.w.lo);                                    \
    }

/* declare 256 interpolation functions (one for each alpha) */ 
BLEND(0)   BLEND(1)   BLEND(2)   BLEND(3) 
BLEND(4)   BLEND(5)   BLEND(6)   BLEND(7) 
BLEND(8)   BLEND(9)   BLEND(10)  BLEND(11) 
BLEND(12)  BLEND(13)  BLEND(14)  BLEND(15) 
BLEND(16)  BLEND(17)  BLEND(18)  BLEND(19) 
BLEND(20)  BLEND(21)  BLEND(22)  BLEND(23) 
BLEND(24)  BLEND(25)  BLEND(26)  BLEND(27) 
BLEND(28)  BLEND(29)  BLEND(30)  BLEND(31) 

/* and list them all in a table */
BLENDER_MAP _trans_blender15 =
{ { 
   (BLENDER_FUNC)blend15_0,   (BLENDER_FUNC)blend15_0,   (BLENDER_FUNC)blend15_0,   (BLENDER_FUNC)blend15_0,
   (BLENDER_FUNC)blend15_0,   (BLENDER_FUNC)blend15_0,   (BLENDER_FUNC)blend15_0,   (BLENDER_FUNC)blend15_0,
   (BLENDER_FUNC)blend15_1,   (BLENDER_FUNC)blend15_1,   (BLENDER_FUNC)blend15_1,   (BLENDER_FUNC)blend15_1,
   (BLENDER_FUNC)blend15_1,   (BLENDER_FUNC)blend15_1,   (BLENDER_FUNC)blend15_1,   (BLENDER_FUNC)blend15_1,
   (BLENDER_FUNC)blend15_2,   (BLENDER_FUNC)blend15_2,   (BLENDER_FUNC)blend15_2,   (BLENDER_FUNC)blend15_2,
   (BLENDER_FUNC)blend15_2,   (BLENDER_FUNC)blend15_2,   (BLENDER_FUNC)blend15_2,   (BLENDER_FUNC)blend15_2,
   (BLENDER_FUNC)blend15_3,   (BLENDER_FUNC)blend15_3,   (BLENDER_FUNC)blend15_3,   (BLENDER_FUNC)blend15_3,
   (BLENDER_FUNC)blend15_3,   (BLENDER_FUNC)blend15_3,   (BLENDER_FUNC)blend15_3,   (BLENDER_FUNC)blend15_3,
   (BLENDER_FUNC)blend15_4,   (BLENDER_FUNC)blend15_4,   (BLENDER_FUNC)blend15_4,   (BLENDER_FUNC)blend15_4,
   (BLENDER_FUNC)blend15_4,   (BLENDER_FUNC)blend15_4,   (BLENDER_FUNC)blend15_4,   (BLENDER_FUNC)blend15_4,
   (BLENDER_FUNC)blend15_5,   (BLENDER_FUNC)blend15_5,   (BLENDER_FUNC)blend15_5,   (BLENDER_FUNC)blend15_5,
   (BLENDER_FUNC)blend15_5,   (BLENDER_FUNC)blend15_5,   (BLENDER_FUNC)blend15_5,   (BLENDER_FUNC)blend15_5,
   (BLENDER_FUNC)blend15_6,   (BLENDER_FUNC)blend15_6,   (BLENDER_FUNC)blend15_6,   (BLENDER_FUNC)blend15_6,
   (BLENDER_FUNC)blend15_6,   (BLENDER_FUNC)blend15_6,   (BLENDER_FUNC)blend15_6,   (BLENDER_FUNC)blend15_6,
   (BLENDER_FUNC)blend15_7,   (BLENDER_FUNC)blend15_7,   (BLENDER_FUNC)blend15_7,   (BLENDER_FUNC)blend15_7,
   (BLENDER_FUNC)blend15_7,   (BLENDER_FUNC)blend15_7,   (BLENDER_FUNC)blend15_7,   (BLENDER_FUNC)blend15_7,
   (BLENDER_FUNC)blend15_8,   (BLENDER_FUNC)blend15_8,   (BLENDER_FUNC)blend15_8,   (BLENDER_FUNC)blend15_8,
   (BLENDER_FUNC)blend15_8,   (BLENDER_FUNC)blend15_8,   (BLENDER_FUNC)blend15_8,   (BLENDER_FUNC)blend15_8,
   (BLENDER_FUNC)blend15_9,   (BLENDER_FUNC)blend15_9,   (BLENDER_FUNC)blend15_9,   (BLENDER_FUNC)blend15_9,
   (BLENDER_FUNC)blend15_9,   (BLENDER_FUNC)blend15_9,   (BLENDER_FUNC)blend15_9,   (BLENDER_FUNC)blend15_9,
   (BLENDER_FUNC)blend15_10,   (BLENDER_FUNC)blend15_10,   (BLENDER_FUNC)blend15_10,   (BLENDER_FUNC)blend15_10,
   (BLENDER_FUNC)blend15_10,   (BLENDER_FUNC)blend15_10,   (BLENDER_FUNC)blend15_10,   (BLENDER_FUNC)blend15_10,
   (BLENDER_FUNC)blend15_11,   (BLENDER_FUNC)blend15_11,   (BLENDER_FUNC)blend15_11,   (BLENDER_FUNC)blend15_11,
   (BLENDER_FUNC)blend15_11,   (BLENDER_FUNC)blend15_11,   (BLENDER_FUNC)blend15_11,   (BLENDER_FUNC)blend15_11,
   (BLENDER_FUNC)blend15_12,   (BLENDER_FUNC)blend15_12,   (BLENDER_FUNC)blend15_12,   (BLENDER_FUNC)blend15_12,
   (BLENDER_FUNC)blend15_12,   (BLENDER_FUNC)blend15_12,   (BLENDER_FUNC)blend15_12,   (BLENDER_FUNC)blend15_12,
   (BLENDER_FUNC)blend15_13,   (BLENDER_FUNC)blend15_13,   (BLENDER_FUNC)blend15_13,   (BLENDER_FUNC)blend15_13,
   (BLENDER_FUNC)blend15_13,   (BLENDER_FUNC)blend15_13,   (BLENDER_FUNC)blend15_13,   (BLENDER_FUNC)blend15_13,
   (BLENDER_FUNC)blend15_14,   (BLENDER_FUNC)blend15_14,   (BLENDER_FUNC)blend15_14,   (BLENDER_FUNC)blend15_14,
   (BLENDER_FUNC)blend15_14,   (BLENDER_FUNC)blend15_14,   (BLENDER_FUNC)blend15_14,   (BLENDER_FUNC)blend15_14,
   (BLENDER_FUNC)blend15_15,   (BLENDER_FUNC)blend15_15,   (BLENDER_FUNC)blend15_15,   (BLENDER_FUNC)blend15_15,
   (BLENDER_FUNC)blend15_15,   (BLENDER_FUNC)blend15_15,   (BLENDER_FUNC)blend15_15,   (BLENDER_FUNC)blend15_15,
   (BLENDER_FUNC)blend15_16,   (BLENDER_FUNC)blend15_16,   (BLENDER_FUNC)blend15_16,   (BLENDER_FUNC)blend15_16,
   (BLENDER_FUNC)blend15_16,   (BLENDER_FUNC)blend15_16,   (BLENDER_FUNC)blend15_16,   (BLENDER_FUNC)blend15_16,
   (BLENDER_FUNC)blend15_17,   (BLENDER_FUNC)blend15_17,   (BLENDER_FUNC)blend15_17,   (BLENDER_FUNC)blend15_17,
   (BLENDER_FUNC)blend15_17,   (BLENDER_FUNC)blend15_17,   (BLENDER_FUNC)blend15_17,   (BLENDER_FUNC)blend15_17,
   (BLENDER_FUNC)blend15_18,   (BLENDER_FUNC)blend15_18,   (BLENDER_FUNC)blend15_18,   (BLENDER_FUNC)blend15_18,
   (BLENDER_FUNC)blend15_18,   (BLENDER_FUNC)blend15_18,   (BLENDER_FUNC)blend15_18,   (BLENDER_FUNC)blend15_18,
   (BLENDER_FUNC)blend15_19,   (BLENDER_FUNC)blend15_19,   (BLENDER_FUNC)blend15_19,   (BLENDER_FUNC)blend15_19,
   (BLENDER_FUNC)blend15_19,   (BLENDER_FUNC)blend15_19,   (BLENDER_FUNC)blend15_19,   (BLENDER_FUNC)blend15_19,
   (BLENDER_FUNC)blend15_20,   (BLENDER_FUNC)blend15_20,   (BLENDER_FUNC)blend15_20,   (BLENDER_FUNC)blend15_20,
   (BLENDER_FUNC)blend15_20,   (BLENDER_FUNC)blend15_20,   (BLENDER_FUNC)blend15_20,   (BLENDER_FUNC)blend15_20,
   (BLENDER_FUNC)blend15_21,   (BLENDER_FUNC)blend15_21,   (BLENDER_FUNC)blend15_21,   (BLENDER_FUNC)blend15_21,
   (BLENDER_FUNC)blend15_21,   (BLENDER_FUNC)blend15_21,   (BLENDER_FUNC)blend15_21,   (BLENDER_FUNC)blend15_21,
   (BLENDER_FUNC)blend15_22,   (BLENDER_FUNC)blend15_22,   (BLENDER_FUNC)blend15_22,   (BLENDER_FUNC)blend15_22,
   (BLENDER_FUNC)blend15_22,   (BLENDER_FUNC)blend15_22,   (BLENDER_FUNC)blend15_22,   (BLENDER_FUNC)blend15_22,
   (BLENDER_FUNC)blend15_23,   (BLENDER_FUNC)blend15_23,   (BLENDER_FUNC)blend15_23,   (BLENDER_FUNC)blend15_23,
   (BLENDER_FUNC)blend15_23,   (BLENDER_FUNC)blend15_23,   (BLENDER_FUNC)blend15_23,   (BLENDER_FUNC)blend15_23,
   (BLENDER_FUNC)blend15_24,   (BLENDER_FUNC)blend15_24,   (BLENDER_FUNC)blend15_24,   (BLENDER_FUNC)blend15_24,
   (BLENDER_FUNC)blend15_24,   (BLENDER_FUNC)blend15_24,   (BLENDER_FUNC)blend15_24,   (BLENDER_FUNC)blend15_24,
   (BLENDER_FUNC)blend15_25,   (BLENDER_FUNC)blend15_25,   (BLENDER_FUNC)blend15_25,   (BLENDER_FUNC)blend15_25,
   (BLENDER_FUNC)blend15_25,   (BLENDER_FUNC)blend15_25,   (BLENDER_FUNC)blend15_25,   (BLENDER_FUNC)blend15_25,
   (BLENDER_FUNC)blend15_26,   (BLENDER_FUNC)blend15_26,   (BLENDER_FUNC)blend15_26,   (BLENDER_FUNC)blend15_26,
   (BLENDER_FUNC)blend15_26,   (BLENDER_FUNC)blend15_26,   (BLENDER_FUNC)blend15_26,   (BLENDER_FUNC)blend15_26,
   (BLENDER_FUNC)blend15_27,   (BLENDER_FUNC)blend15_27,   (BLENDER_FUNC)blend15_27,   (BLENDER_FUNC)blend15_27,
   (BLENDER_FUNC)blend15_27,   (BLENDER_FUNC)blend15_27,   (BLENDER_FUNC)blend15_27,   (BLENDER_FUNC)blend15_27,
   (BLENDER_FUNC)blend15_28,   (BLENDER_FUNC)blend15_28,   (BLENDER_FUNC)blend15_28,   (BLENDER_FUNC)blend15_28,
   (BLENDER_FUNC)blend15_28,   (BLENDER_FUNC)blend15_28,   (BLENDER_FUNC)blend15_28,   (BLENDER_FUNC)blend15_28,
   (BLENDER_FUNC)blend15_29,   (BLENDER_FUNC)blend15_29,   (BLENDER_FUNC)blend15_29,   (BLENDER_FUNC)blend15_29,
   (BLENDER_FUNC)blend15_29,   (BLENDER_FUNC)blend15_29,   (BLENDER_FUNC)blend15_29,   (BLENDER_FUNC)blend15_29,
   (BLENDER_FUNC)blend15_30,   (BLENDER_FUNC)blend15_30,   (BLENDER_FUNC)blend15_30,   (BLENDER_FUNC)blend15_30,
   (BLENDER_FUNC)blend15_30,   (BLENDER_FUNC)blend15_30,   (BLENDER_FUNC)blend15_30,   (BLENDER_FUNC)blend15_30,
   (BLENDER_FUNC)blend15_31,   (BLENDER_FUNC)blend15_31,   (BLENDER_FUNC)blend15_31,   (BLENDER_FUNC)blend15_31,
   (BLENDER_FUNC)blend15_31,   (BLENDER_FUNC)blend15_31,   (BLENDER_FUNC)blend15_31,   (BLENDER_FUNC)blend15_31,
} };



#endif
