/*         ______   ___    ___ 
 *        /\  _  \ /\_ \  /\_ \ 
 *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___ 
 *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
 *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
 *           \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/
 *            \/_/\/_/\/____/\/____/\/____/\/___L\ \/_/ \/___/
 *                                           /\____/
 *                                           \_/__/
 *      By Shawn Hargreaves,
 *      1 Salisbury Road,
 *      Market Drayton,
 *      Shropshire,
 *      England, TF9 1AJ.
 *
 *      Interpolation routines for 16 bit truecolor pixels.
 *
 *      See readme.txt for copyright information.
 */


#include "allegro.h"

#ifdef ALLEGRO_COLOR16

union dword { unsigned long l; struct { unsigned short lo,hi; } w; };

/* macro for constructing the blender routines */
#define BLEND(n)                                                         \
                                         \
    static unsigned long blend16_##n(union dword x, union dword y)       \
    {                                                                    \
    const unsigned long mask=0x7e0f81f;                                  \
    union dword result;                                                  \
    x.w.hi=x.w.lo,y.w.hi=y.w.lo;                                         \
    result.l=(((x.l&mask)*(n*32/31)+(y.l&mask)*(32-(n*32/31)))/32)&mask; \
    return (result.w.hi|result.w.lo);                                    \
    }

/* declare 256 interpolation functions (one for each alpha) */ 
BLEND(0)   BLEND(1)   BLEND(2)   BLEND(3) 
BLEND(4)   BLEND(5)   BLEND(6)   BLEND(7) 
BLEND(8)   BLEND(9)   BLEND(10)  BLEND(11) 
BLEND(12)  BLEND(13)  BLEND(14)  BLEND(15) 
BLEND(16)  BLEND(17)  BLEND(18)  BLEND(19) 
BLEND(20)  BLEND(21)  BLEND(22)  BLEND(23) 
BLEND(24)  BLEND(25)  BLEND(26)  BLEND(27) 
BLEND(28)  BLEND(29)  BLEND(30)  BLEND(31) 

/* and list them all in a table */
BLENDER_MAP _trans_blender16 =
{ { 
   (BLENDER_FUNC)blend16_0,   (BLENDER_FUNC)blend16_0,   (BLENDER_FUNC)blend16_0,   (BLENDER_FUNC)blend16_0,
   (BLENDER_FUNC)blend16_0,   (BLENDER_FUNC)blend16_0,   (BLENDER_FUNC)blend16_0,   (BLENDER_FUNC)blend16_0,
   (BLENDER_FUNC)blend16_1,   (BLENDER_FUNC)blend16_1,   (BLENDER_FUNC)blend16_1,   (BLENDER_FUNC)blend16_1,
   (BLENDER_FUNC)blend16_1,   (BLENDER_FUNC)blend16_1,   (BLENDER_FUNC)blend16_1,   (BLENDER_FUNC)blend16_1,
   (BLENDER_FUNC)blend16_2,   (BLENDER_FUNC)blend16_2,   (BLENDER_FUNC)blend16_2,   (BLENDER_FUNC)blend16_2,
   (BLENDER_FUNC)blend16_2,   (BLENDER_FUNC)blend16_2,   (BLENDER_FUNC)blend16_2,   (BLENDER_FUNC)blend16_2,
   (BLENDER_FUNC)blend16_3,   (BLENDER_FUNC)blend16_3,   (BLENDER_FUNC)blend16_3,   (BLENDER_FUNC)blend16_3,
   (BLENDER_FUNC)blend16_3,   (BLENDER_FUNC)blend16_3,   (BLENDER_FUNC)blend16_3,   (BLENDER_FUNC)blend16_3,
   (BLENDER_FUNC)blend16_4,   (BLENDER_FUNC)blend16_4,   (BLENDER_FUNC)blend16_4,   (BLENDER_FUNC)blend16_4,
   (BLENDER_FUNC)blend16_4,   (BLENDER_FUNC)blend16_4,   (BLENDER_FUNC)blend16_4,   (BLENDER_FUNC)blend16_4,
   (BLENDER_FUNC)blend16_5,   (BLENDER_FUNC)blend16_5,   (BLENDER_FUNC)blend16_5,   (BLENDER_FUNC)blend16_5,
   (BLENDER_FUNC)blend16_5,   (BLENDER_FUNC)blend16_5,   (BLENDER_FUNC)blend16_5,   (BLENDER_FUNC)blend16_5,
   (BLENDER_FUNC)blend16_6,   (BLENDER_FUNC)blend16_6,   (BLENDER_FUNC)blend16_6,   (BLENDER_FUNC)blend16_6,
   (BLENDER_FUNC)blend16_6,   (BLENDER_FUNC)blend16_6,   (BLENDER_FUNC)blend16_6,   (BLENDER_FUNC)blend16_6,
   (BLENDER_FUNC)blend16_7,   (BLENDER_FUNC)blend16_7,   (BLENDER_FUNC)blend16_7,   (BLENDER_FUNC)blend16_7,
   (BLENDER_FUNC)blend16_7,   (BLENDER_FUNC)blend16_7,   (BLENDER_FUNC)blend16_7,   (BLENDER_FUNC)blend16_7,
   (BLENDER_FUNC)blend16_8,   (BLENDER_FUNC)blend16_8,   (BLENDER_FUNC)blend16_8,   (BLENDER_FUNC)blend16_8,
   (BLENDER_FUNC)blend16_8,   (BLENDER_FUNC)blend16_8,   (BLENDER_FUNC)blend16_8,   (BLENDER_FUNC)blend16_8,
   (BLENDER_FUNC)blend16_9,   (BLENDER_FUNC)blend16_9,   (BLENDER_FUNC)blend16_9,   (BLENDER_FUNC)blend16_9,
   (BLENDER_FUNC)blend16_9,   (BLENDER_FUNC)blend16_9,   (BLENDER_FUNC)blend16_9,   (BLENDER_FUNC)blend16_9,
   (BLENDER_FUNC)blend16_10,   (BLENDER_FUNC)blend16_10,   (BLENDER_FUNC)blend16_10,   (BLENDER_FUNC)blend16_10,
   (BLENDER_FUNC)blend16_10,   (BLENDER_FUNC)blend16_10,   (BLENDER_FUNC)blend16_10,   (BLENDER_FUNC)blend16_10,
   (BLENDER_FUNC)blend16_11,   (BLENDER_FUNC)blend16_11,   (BLENDER_FUNC)blend16_11,   (BLENDER_FUNC)blend16_11,
   (BLENDER_FUNC)blend16_11,   (BLENDER_FUNC)blend16_11,   (BLENDER_FUNC)blend16_11,   (BLENDER_FUNC)blend16_11,
   (BLENDER_FUNC)blend16_12,   (BLENDER_FUNC)blend16_12,   (BLENDER_FUNC)blend16_12,   (BLENDER_FUNC)blend16_12,
   (BLENDER_FUNC)blend16_12,   (BLENDER_FUNC)blend16_12,   (BLENDER_FUNC)blend16_12,   (BLENDER_FUNC)blend16_12,
   (BLENDER_FUNC)blend16_13,   (BLENDER_FUNC)blend16_13,   (BLENDER_FUNC)blend16_13,   (BLENDER_FUNC)blend16_13,
   (BLENDER_FUNC)blend16_13,   (BLENDER_FUNC)blend16_13,   (BLENDER_FUNC)blend16_13,   (BLENDER_FUNC)blend16_13,
   (BLENDER_FUNC)blend16_14,   (BLENDER_FUNC)blend16_14,   (BLENDER_FUNC)blend16_14,   (BLENDER_FUNC)blend16_14,
   (BLENDER_FUNC)blend16_14,   (BLENDER_FUNC)blend16_14,   (BLENDER_FUNC)blend16_14,   (BLENDER_FUNC)blend16_14,
   (BLENDER_FUNC)blend16_15,   (BLENDER_FUNC)blend16_15,   (BLENDER_FUNC)blend16_15,   (BLENDER_FUNC)blend16_15,
   (BLENDER_FUNC)blend16_15,   (BLENDER_FUNC)blend16_15,   (BLENDER_FUNC)blend16_15,   (BLENDER_FUNC)blend16_15,
   (BLENDER_FUNC)blend16_16,   (BLENDER_FUNC)blend16_16,   (BLENDER_FUNC)blend16_16,   (BLENDER_FUNC)blend16_16,
   (BLENDER_FUNC)blend16_16,   (BLENDER_FUNC)blend16_16,   (BLENDER_FUNC)blend16_16,   (BLENDER_FUNC)blend16_16,
   (BLENDER_FUNC)blend16_17,   (BLENDER_FUNC)blend16_17,   (BLENDER_FUNC)blend16_17,   (BLENDER_FUNC)blend16_17,
   (BLENDER_FUNC)blend16_17,   (BLENDER_FUNC)blend16_17,   (BLENDER_FUNC)blend16_17,   (BLENDER_FUNC)blend16_17,
   (BLENDER_FUNC)blend16_18,   (BLENDER_FUNC)blend16_18,   (BLENDER_FUNC)blend16_18,   (BLENDER_FUNC)blend16_18,
   (BLENDER_FUNC)blend16_18,   (BLENDER_FUNC)blend16_18,   (BLENDER_FUNC)blend16_18,   (BLENDER_FUNC)blend16_18,
   (BLENDER_FUNC)blend16_19,   (BLENDER_FUNC)blend16_19,   (BLENDER_FUNC)blend16_19,   (BLENDER_FUNC)blend16_19,
   (BLENDER_FUNC)blend16_19,   (BLENDER_FUNC)blend16_19,   (BLENDER_FUNC)blend16_19,   (BLENDER_FUNC)blend16_19,
   (BLENDER_FUNC)blend16_20,   (BLENDER_FUNC)blend16_20,   (BLENDER_FUNC)blend16_20,   (BLENDER_FUNC)blend16_20,
   (BLENDER_FUNC)blend16_20,   (BLENDER_FUNC)blend16_20,   (BLENDER_FUNC)blend16_20,   (BLENDER_FUNC)blend16_20,
   (BLENDER_FUNC)blend16_21,   (BLENDER_FUNC)blend16_21,   (BLENDER_FUNC)blend16_21,   (BLENDER_FUNC)blend16_21,
   (BLENDER_FUNC)blend16_21,   (BLENDER_FUNC)blend16_21,   (BLENDER_FUNC)blend16_21,   (BLENDER_FUNC)blend16_21,
   (BLENDER_FUNC)blend16_22,   (BLENDER_FUNC)blend16_22,   (BLENDER_FUNC)blend16_22,   (BLENDER_FUNC)blend16_22,
   (BLENDER_FUNC)blend16_22,   (BLENDER_FUNC)blend16_22,   (BLENDER_FUNC)blend16_22,   (BLENDER_FUNC)blend16_22,
   (BLENDER_FUNC)blend16_23,   (BLENDER_FUNC)blend16_23,   (BLENDER_FUNC)blend16_23,   (BLENDER_FUNC)blend16_23,
   (BLENDER_FUNC)blend16_23,   (BLENDER_FUNC)blend16_23,   (BLENDER_FUNC)blend16_23,   (BLENDER_FUNC)blend16_23,
   (BLENDER_FUNC)blend16_24,   (BLENDER_FUNC)blend16_24,   (BLENDER_FUNC)blend16_24,   (BLENDER_FUNC)blend16_24,
   (BLENDER_FUNC)blend16_24,   (BLENDER_FUNC)blend16_24,   (BLENDER_FUNC)blend16_24,   (BLENDER_FUNC)blend16_24,
   (BLENDER_FUNC)blend16_25,   (BLENDER_FUNC)blend16_25,   (BLENDER_FUNC)blend16_25,   (BLENDER_FUNC)blend16_25,
   (BLENDER_FUNC)blend16_25,   (BLENDER_FUNC)blend16_25,   (BLENDER_FUNC)blend16_25,   (BLENDER_FUNC)blend16_25,
   (BLENDER_FUNC)blend16_26,   (BLENDER_FUNC)blend16_26,   (BLENDER_FUNC)blend16_26,   (BLENDER_FUNC)blend16_26,
   (BLENDER_FUNC)blend16_26,   (BLENDER_FUNC)blend16_26,   (BLENDER_FUNC)blend16_26,   (BLENDER_FUNC)blend16_26,
   (BLENDER_FUNC)blend16_27,   (BLENDER_FUNC)blend16_27,   (BLENDER_FUNC)blend16_27,   (BLENDER_FUNC)blend16_27,
   (BLENDER_FUNC)blend16_27,   (BLENDER_FUNC)blend16_27,   (BLENDER_FUNC)blend16_27,   (BLENDER_FUNC)blend16_27,
   (BLENDER_FUNC)blend16_28,   (BLENDER_FUNC)blend16_28,   (BLENDER_FUNC)blend16_28,   (BLENDER_FUNC)blend16_28,
   (BLENDER_FUNC)blend16_28,   (BLENDER_FUNC)blend16_28,   (BLENDER_FUNC)blend16_28,   (BLENDER_FUNC)blend16_28,
   (BLENDER_FUNC)blend16_29,   (BLENDER_FUNC)blend16_29,   (BLENDER_FUNC)blend16_29,   (BLENDER_FUNC)blend16_29,
   (BLENDER_FUNC)blend16_29,   (BLENDER_FUNC)blend16_29,   (BLENDER_FUNC)blend16_29,   (BLENDER_FUNC)blend16_29,
   (BLENDER_FUNC)blend16_30,   (BLENDER_FUNC)blend16_30,   (BLENDER_FUNC)blend16_30,   (BLENDER_FUNC)blend16_30,
   (BLENDER_FUNC)blend16_30,   (BLENDER_FUNC)blend16_30,   (BLENDER_FUNC)blend16_30,   (BLENDER_FUNC)blend16_30,
   (BLENDER_FUNC)blend16_31,   (BLENDER_FUNC)blend16_31,   (BLENDER_FUNC)blend16_31,   (BLENDER_FUNC)blend16_31,
   (BLENDER_FUNC)blend16_31,   (BLENDER_FUNC)blend16_31,   (BLENDER_FUNC)blend16_31,   (BLENDER_FUNC)blend16_31,
} };



#endif
